@@ -31,7 +31,7 @@ DATETIME_COL_FMT = Literal['DateTime', 'Date', 'Time']
 #  'Split year and month in 2 columns',
 #  'Start and End Date in 2 columns']
 
-EXPECTED_DELIVERY = Literal['15min', 'Hourly', 'Daily', 'Weekly', 'Monthly', 'Quarterly']
+EXPECTED_DELIVERY = Literal['5min', '15min', 'Hourly', 'Daily', 'Weekly', 'Monthly', 'Quarterly']
 
 # deploy_type 'File' for (FTP. Email, Upload)
 DEPLOY_TYPE = Literal['Email', 'Ftp', 'Upload', 'Timer']
