@@ -92,7 +92,7 @@ ApiInputs = namedtuple('ApiInputs',
 
 DiscoveryIntegrationInput = namedtuple(
     'DiscoveryIntegrationInput',
-    ['api_project_id', 'installation_id', 'network_device_id', 'integration_device_id', 'user_id', 'batch_no'])
+    ['api_project_id', 'installation_id', 'network_device_id', 'integration_device_id', 'user_id', 'batch_no', 'job_id'])
 # DiscoveryIntegrationInput.__doc__ = """DiscoveryIntegrationInput(api_project_id, installation_id, network_device_id, integration_device_id, user_id, batch_no)
 #
 # Defines the required inputs to be provided to the run_discovery() method's integration_input parameter.
