@@ -7,7 +7,6 @@
 A module for .....
 """
 
-from .analytics import (get_parent_modules_list, get_clone_modules_list, run_clone_modules,
-                                            sensor_has_not_changed)
+from .analytics import (get_parent_modules_list, get_clone_modules_list, sensor_has_not_changed)  ## , run_clone_modules
 
-__all__ = ['get_parent_modules_list', 'get_clone_modules_list', 'run_clone_modules', 'sensor_has_not_changed']
+__all__ = ['get_parent_modules_list', 'get_clone_modules_list', 'sensor_has_not_changed']  # 'run_clone_modules',
