@@ -126,7 +126,7 @@ def _timezone_offsets(api_inputs: ApiInputs, date_from: datetime.date, date_to:
 
 
 def _timezone_dst_offsets(api_inputs: ApiInputs, date_from: datetime.date, date_to: datetime.date,
-                          installation_id_list: list):
+                          installation_id_list: list = None, timezone_name: str = None):
     """Get timezones
 
     Parameters
@@ -139,6 +139,9 @@ def _timezone_dst_offsets(api_inputs: ApiInputs, date_from: datetime.date, date_
         Last (latest) record
     installation_id_list : list
         List of InstallationIDs
+    timezone_name: str
+        Specific timezone name to retrieve DST offsets.
+        Defaults to None.
 
     Returns
     -------
@@ -150,16 +153,24 @@ def _timezone_dst_offsets(api_inputs: ApiInputs, date_from: datetime.date, date_
     payload = {
         "dateFrom": date_from.isoformat(),
         "dateTo": date_to.isoformat(),
-        "installationIds": installation_id_list
+        "installationIds": installation_id_list,
+        "timezoneName": timezone_name
     }
 
     if api_inputs.api_base_url == '' or api_inputs.bearer_token == '':
         logger.error("You must call initialize() before using API.")
         return pandas.DataFrame()
 
+    if timezone_name == '':
+        logger.error("timezone_name parameter cannot be an empty string.")
+        return pandas.DataFrame()
+
     headers = api_inputs.api_headers.default
 
     url = f"{api_inputs.api_base_url}/timezones/dst-transitions"
+    if not timezone_name is None:
+        url = f"{api_inputs.api_base_url}/timezones/name/dst-transitions"
+
     logger.info("Sending request: POST %s", url)
 
     try:
@@ -180,12 +191,14 @@ def _timezone_dst_offsets(api_inputs: ApiInputs, date_from: datetime.date, date_
         timezone_offsets = timezone_offsets.explode('dstIntervals')
         timezone_offsets = timezone_offsets.reset_index(drop=True).join(timezone_offsets.reset_index(drop=True)
                                                                         .dstIntervals.apply(pandas.Series)).drop(columns=['dstIntervals', 'timezoneName'])
-        return site_timezones.merge(timezone_offsets, on='timezoneId')
+        if timezone_name is None:
+            return site_timezones.merge(timezone_offsets, on='timezoneId')
+        else:
+            return timezone_offsets
     except Exception as ex:
         logger.error("API Call was not successful.", ex)
         return pandas.DataFrame()
 
-
 def _upsert_entities_affected_count(api_inputs: ApiInputs, entities_affected_count: int):
     """Updates data feed and data feed file status entities affected count.
 
