@@ -206,7 +206,7 @@ def get_sites(api_inputs: ApiInputs, include_tag_groups: Union[list, bool] = Fal
         logger.error('No data returned for this API call. %s', response.request.url)
         return response_status
 
-    df = pandas.read_json(response.text)
+    df = pandas.read_json(response.text, dtype={'InstallationCode': str})
 
     return df
 
@@ -641,7 +641,7 @@ def get_equipment_classes(api_inputs: ApiInputs):
         logger.error("You must call initialize() before using API.")
         return pandas.DataFrame()
 
-    response = requests.request("GET", url, timeout=20, headers=headers, verify=False)
+    response = requests.request("GET", url, timeout=20, headers=headers)
     response_status = '{} {}'.format(response.status_code, response.reason)
     if response.status_code != 200:
         logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
@@ -656,6 +656,43 @@ def get_equipment_classes(api_inputs: ApiInputs):
 
     return df
 
+
+def get_timezones(api_inputs: ApiInputs):
+    """Get timezones
+
+    Parameters
+    ----------
+    api_inputs : ApiInputs
+        Object returned by initialize() function.
+
+    Returns
+    -------
+    df : pandas.DataFrame
+    """
+    headers = api_inputs.api_headers.default
+
+    if api_inputs.api_base_url == '' or api_inputs.bearer_token == '':
+        logger.error("You must call initialize() before using API.")
+        return pandas.DataFrame()
+
+    url = f"{api_inputs.api_base_url}/timezones/all"
+    logger.info("Sending request: GET %s", url)
+
+    response = requests.request("GET", url, timeout=20, headers=headers)
+    response_status = '{} {}'.format(response.status_code, response.reason)
+    if response.status_code != 200:
+        logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
+                     response.reason)
+        return response_status, pandas.DataFrame()
+    elif len(response.text) == 0:
+        logger.error('No data returned for this API call. %s', response.request.url)
+        return response_status, pandas.DataFrame()
+
+    df = pandas.read_json(response.text)
+    df.columns = _column_name_cap(df.columns)
+
+    return df
+
 # def get_portfolios(api_inputs: ApiInputs, search_term: str = None):
 #     """Get list of units of measure by type.
 #
