@@ -9,8 +9,10 @@ A module for integrating asset creation, asset updates, data ingestion, etc into
 import gc
 import re
 import sys
+from typing import Union
 import pandas
 import pandas as pd
+from ..integration.helpers import get_timezones
 import pandera
 import requests
 import datetime
@@ -33,7 +35,7 @@ consoleHandler.setFormatter(formatter)
 
 
 @_with_func_attrs(df_required_columns=['ApiProjectId', 'InstallationId', 'NetworkDeviceId', 'UserId', 'BatchNo',
-                                       'DriverUniqueId', 'Timestamp', 'DiscoveredValue', 'DriverClassName'],
+                                       'DriverUniqueId', 'Timestamp', 'DiscoveredValue', 'DriverClassName', 'JobId'],
                   df_optional_columns=['DriverDeviceType', 'ObjectPropertyTemplateName', 'UnitOfMeasureAbbrev',
                                        'DeviceName', 'DisplayName', 'EquipmentType', 'EquipmentLabel', ])
 def upsert_discovered_records(df: pandas.DataFrame, api_inputs: ApiInputs, discovery_properties_columns: list,
@@ -73,7 +75,7 @@ def upsert_discovered_records(df: pandas.DataFrame, api_inputs: ApiInputs, disco
     data_frame = df.copy()
 
     required_columns = ['ApiProjectId', 'InstallationId', 'NetworkDeviceId', 'UserId', 'BatchNo', 'DriverUniqueId',
-                        'Timestamp', 'DiscoveredValue', 'DriverClassName']
+                        'Timestamp', 'DiscoveredValue', 'DriverClassName', 'JobId']
     optional_columns = ['DriverDeviceType', 'ObjectPropertyTemplateName', 'UnitOfMeasureAbbrev', 'DeviceName',
                         'DisplayName', 'EquipmentType', 'EquipmentLabel', ]
     proposed_columns = data_frame.columns.tolist()
@@ -164,7 +166,7 @@ def upsert_discovered_records(df: pandas.DataFrame, api_inputs: ApiInputs, disco
     final_req_cols = ['ApiProjectId', 'InstallationId', 'NetworkDeviceId', 'DriverClassName', 'UserId', 'BatchNo',
                       'DriverUniqueId', 'CurrentValue', 'DriverDeviceType', 'ObjectPropertyTemplateName',
                       'UnitOfMeasureAbbrev', 'DeviceName', 'DisplayName', 'EquipmentType', 'EquipmentLabel',
-                      'DeviceTags', 'SensorTags', 'OntologyTags', 'Metadata', 'DiscoveryProperties']
+                      'DeviceTags', 'SensorTags', 'OntologyTags', 'Metadata', 'DiscoveryProperties', 'JobId']
 
     if set(data_frame.columns.tolist()).issubset(final_req_cols):
         batch_size = 50
@@ -179,7 +181,7 @@ def upsert_discovered_records(df: pandas.DataFrame, api_inputs: ApiInputs, disco
 
         for name, group in grouped_df:
             discovery_payload = group.groupby(
-                by=['ApiProjectId', 'InstallationId', 'NetworkDeviceId', 'DriverClassName', 'UserId', 'BatchNo']).apply(
+                by=['ApiProjectId', 'InstallationId', 'NetworkDeviceId', 'DriverClassName', 'UserId', 'BatchNo', 'JobId']).apply(
                 lambda x: x[['DriverUniqueId', 'CurrentValue', 'DriverDeviceType', 'ObjectPropertyTemplateName',
                              'UnitOfMeasureAbbrev', 'DeviceName', 'DisplayName', 'EquipmentType', 'EquipmentLabel',
                              'DeviceTags', 'SensorTags', 'OntologyTags', 'Metadata', 'DiscoveryProperties']].to_dict(
@@ -854,7 +856,8 @@ def upsert_timeseries_ds(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_t
 
 @_with_func_attrs(df_required_columns=['ObjectPropertyId', 'InstallationId', 'Timestamp', 'Value'])
 def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time: bool = True,
-                      save_additional_columns_as_slices: bool = False, data_feed_file_status_id: uuid.UUID = None):
+                      save_additional_columns_as_slices: bool = False, data_feed_file_status_id: uuid.UUID = None, 
+                      is_specific_timezone: Union[bool, str] = False):
     """Upserts timeseries to EventHub for processing.
 
     The following columns are required to be present in the data_frame:
@@ -870,7 +873,8 @@ def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time
     api_inputs: ApiInputs
         Object returned by initialize() function.
     is_local_time : bool, default = True
-         Whether the datetime values are in local time or UTC. If false, then UTC (Default value = True).
+         Whether the datetime values are in local time or UTC. If, False and is_specific_timezone is False, then UTC (Default value = True).
+         Should be set to False when 'is_specific_timezone' has value.
     save_additional_columns_as_slices : bool, default = False
          (Default value = False)
     data_feed_file_status_id : uuid.UUID, default = None
@@ -879,6 +883,10 @@ def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time
 
          Once deployed, the DataFeedFileStatusId field will contain a unique Guid which will assist in
          tracking upload results and logging.
+    is_specific_timezone : Union[False, str]
+        Accepts a timezone name as the specific timezone used by the source data. Defaults to False.
+        Cannot have value if 'is_local_time' is set to True.
+        Retrieve list of timezones using 'sw.integration.get_timezones()'
 
     Returns
     -------
@@ -889,19 +897,61 @@ def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time
     """
 
     data_frame = df.copy()
+    timezones_df = pd.DataFrame()
     logger.info(f'Data frame size: {convert_bytes(data_frame.memory_usage(deep=True).sum())}')
 
     if api_inputs.api_base_url == '' or api_inputs.bearer_token == '':
         logger.error("You must call initialize() before using API.")
         return pandas.DataFrame()
 
+    if not isinstance(is_specific_timezone, str) and not isinstance(is_specific_timezone, bool):
+        logger.error("'is_specific_timezone' parameter can only be of type str or bool.")
+        return pandas.DataFrame()
+
+    if is_specific_timezone != False and is_local_time == True:
+        logger.error("Assigning specific timezone is only possible if 'is_local_time' is set to False.")
+        return pandas.DataFrame()
+
+    if is_specific_timezone == True:
+        logger.error("'is_specific_timezone' parameter value, if not False, should be a valid timezone. Retrieve list of timezones using 'sw.integration.get_timezones()")
+        return pandas.DataFrame()
+
+    if is_specific_timezone == '':
+        logger.error("'is_specific_timezone' cannot be an empty string. Retrieve list of timezones using 'sw.integration.get_timezones()")
+        return pandas.DataFrame()
+
+    is_specific_timezone_conversion = False
+    if is_specific_timezone != False and (isinstance(is_specific_timezone, str) and is_specific_timezone != ''):
+        timezone_name_df = get_timezones(api_inputs=api_inputs)
+
+        if timezone_name_df is None or timezone_name_df.__len__() == 0:
+            logger.error("Failed to retrieve the allowed Timezone names.")
+            return pandas.DataFrame()
+        
+        if not is_specific_timezone in timezone_name_df.TimezoneName.values:
+            logger.error(f"'is_specific_timezone' parameter value '{is_specific_timezone}' is not found in Timezone Names list.")
+            logger.info(f"'Timezone name list can be retrieved using 'sw.integration.get_timezones()' method.")
+            return pandas.DataFrame()
+        
+        is_specific_timezone_conversion = True
+
     required_columns = ['ObjectPropertyId', 'InstallationId', 'Timestamp', 'Value']
     req_cols = ', '.join(required_columns)
     proposed_columns = data_frame.columns.tolist()
 
     if not set(required_columns).issubset(proposed_columns):
         logger.exception('Missing required column(s): %s', set(required_columns).difference(proposed_columns))
-        return 'integration.upsert_timeseries_ds() - data_frame must contain the following columns: ' + req_cols
+        return 'integration.upsert_timeseries() - data_frame must contain the following columns: ' + req_cols
+
+    if not pandas.api.types.is_datetime64_ns_dtype(data_frame['Timestamp']):
+        logger.exception("Timestamp series value is not of type: datetime64[ns] dtype.")
+        return "Timestamp series value is not of type: datetime64[ns] dtype."
+
+    null_count_timestamp = data_frame['Timestamp'].isna().sum()
+    if null_count_timestamp > 0:
+        logger.warning(f"There are {null_count_timestamp} records with a null or empty Timestamp value. " \
+            f"These are being dropped resulting in {data_frame.shape[0] - null_count_timestamp} records will be upserted.")
+        data_frame = data_frame.dropna(subset=['Timestamp'])
 
     slice_columns = set(proposed_columns).difference(set(required_columns))
 
@@ -934,20 +984,11 @@ def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time
         data_feed_file_status_id = uuid.uuid4()
         data_frame['DataFeedFileStatusId'] = data_feed_file_status_id
 
-    if not pandas.api.types.is_datetime64_ns_dtype(data_frame['Timestamp']):
-        logger.exception("Timestamp series value is not of type: datetime64[ns] dtype.")
-        return "Timestamp series value is not of type: datetime64[ns] dtype."
-
-    site_list = data_frame['InstallationId'].unique().tolist()
+    
     start_date = data_frame['Timestamp'].min(axis=0, skipna=True)
     end_date = data_frame['Timestamp'].max(axis=0, skipna=True)
 
-    timezones_df = _timezone_dst_offsets(api_inputs=api_inputs, date_from=start_date.date(), date_to=end_date.date(),
-                                         installation_id_list=site_list)
-
-    if timezones_df.empty:
-        logger.exception('Timezone DST offsets failed to retrieve.')
-        return 'Timezone DST offsets failed to retrieve.'
+    specific_timezone_df = pd.DataFrame()
 
     def convert_dst_interval_dates(row):
         row['start'] = pd.to_datetime(row['start'])
@@ -955,6 +996,46 @@ def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time
         row['offsetToUtcMinutes'] = row['standardOffsetUtc'] + row['dstOffsetUtc']
         return row
 
+    if is_specific_timezone_conversion:
+        def timezone_to_utc(row):
+            if row['offsetToUtcMinutes'] >= 0:
+                j_row = row['Timestamp'] - datetime.timedelta(minutes=row['offsetToUtcMinutes'])
+            else:
+                j_row = row['Timestamp'] + datetime.timedelta(minutes=abs(row['offsetToUtcMinutes']))
+            return j_row
+
+        specific_timezone_df = _timezone_dst_offsets(api_inputs=api_inputs, date_from=start_date.date(), date_to=end_date.date(), timezone_name=is_specific_timezone)
+
+        if specific_timezone_df.empty:
+            logger.exception('Failed to retrieve the specific timezone offsets.')
+            return 'Failed to retrieve the specific timezone offsets.'
+
+        specific_timezone_df = specific_timezone_df.apply(convert_dst_interval_dates, axis=1)
+        specific_timezone_df = specific_timezone_df.drop(columns=['timezoneId', 'standardOffsetUtc', 'dstOffsetUtc'])
+        
+        data_frame['year'] = pd.DatetimeIndex(data_frame['Timestamp']).year
+        data_frame = data_frame.merge(specific_timezone_df, left_on='year', right_on='year', how='inner')
+
+        del specific_timezone_df
+        gc.collect()
+
+        data_frame = data_frame[(data_frame.Timestamp >= data_frame.start) & (data_frame.Timestamp < data_frame.end) & (data_frame.Timestamp.apply(lambda x: x.year) == data_frame.year)]
+        data_frame = data_frame.drop(columns=['start', 'end', 'year'])
+        
+        data_frame['Timestamp'] = data_frame.apply(timezone_to_utc, axis=1)
+        data_frame = data_frame.drop(columns=['offsetToUtcMinutes'])
+
+        # Set automatically to is_local_time = False 
+        is_local_time = False
+
+    site_list = data_frame['InstallationId'].unique().tolist()
+    timezones_df = _timezone_dst_offsets(api_inputs=api_inputs, date_from=start_date.date(), date_to=end_date.date(),
+                                         installation_id_list=site_list)
+
+    if timezones_df.empty:
+        logger.exception('Timezone DST offsets failed to retrieve.')
+        return 'Timezone DST offsets failed to retrieve.'
+
     timezones_df = timezones_df.apply(convert_dst_interval_dates, axis=1)
 
     if not is_local_time:
@@ -978,11 +1059,11 @@ def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time
 
     logger.info('Merging data frame input with timezone installation dst offsets.')
     data_frame = data_frame.merge(timezones_df, left_on='InstallationId', right_on='installationId', how='inner')
-
+    
     del timezones_df
     gc.collect()
 
-    data_frame = data_frame[(data_frame.Timestamp >= data_frame.start) & (data_frame.Timestamp <= data_frame.end) & (data_frame.Timestamp.apply(lambda x: x.year) == data_frame.year)]
+    data_frame = data_frame[(data_frame.Timestamp >= data_frame.start) & (data_frame.Timestamp < data_frame.end) & (data_frame.Timestamp.apply(lambda x: x.year) == data_frame.year)]
     data_frame = data_frame.drop(columns=['start', 'end', 'year'])
 
     if is_local_time:
@@ -1040,7 +1121,6 @@ def upsert_timeseries(df: pandas.DataFrame, api_inputs: ApiInputs, is_local_time
     name = 'Timeseries'
     
     logger.info(f'Data frame size: {convert_bytes(data_frame.memory_usage(deep=True).sum())}')
-
     upload_result = Blob.upload(api_inputs=api_inputs, data_frame=data_frame, container=container, folder=folder, name=name, batch_id=data_feed_file_status_id, include_header=True)
 
     json_payload = {"path": upload_result[0], "fileCount": upload_result[1]}
@@ -1349,7 +1429,7 @@ def upsert_event_work_order_id(api_inputs: ApiInputs, event_task_id: uuid.UUID,
 
     url = f"{api_inputs.api_projects_endpoint}/{api_inputs.api_project_id}/events/{str(event_task_id)}/work-order"
 
-    response = requests.put(url=url, data=payload, headers=header)
+    response = requests.put(url=url, json=payload, headers=header)
     response_status = '{} {}'.format(response.status_code, response.reason)
     if response.status_code != 200:
         logger.error("API Call was not successful. Response Status: %s. Reason: %s.", response.status_code,
@@ -1358,6 +1438,7 @@ def upsert_event_work_order_id(api_inputs: ApiInputs, event_task_id: uuid.UUID,
 
     return response_status, response.text
 
+
 @_with_func_attrs(df_required_columns=['Identifier'])
 def upsert_tags(api_inputs: ApiInputs, df: pandas.DataFrame, tag_level: TAG_LEVEL):
     """
@@ -1433,14 +1514,8 @@ def upsert_tags(api_inputs: ApiInputs, df: pandas.DataFrame, tag_level: TAG_LEVE
 
     tag_level_identifier_id = switcher_tag_level_identifier.get(tag_level)
 
-    required_columns = ['TagsJson']
-    required_columns.append(tag_level_identifier_id)
-    
     data_frame.rename(columns = {'Identifier':tag_level_identifier_id}, inplace = True)
 
-    # remove not required columns
-    data_frame = data_frame.loc[:, required_columns]
-
     # action request
     headers = api_inputs.api_headers.integration
 
@@ -1469,6 +1544,7 @@ def upsert_tags(api_inputs: ApiInputs, df: pandas.DataFrame, tag_level: TAG_LEVE
 
     return response_df
 
+
 @_with_func_attrs(df_required_columns=['DeviceId'])
 def upsert_device_metadata(api_inputs: ApiInputs, df: pandas.DataFrame):
     """
@@ -1525,11 +1601,6 @@ def upsert_device_metadata(api_inputs: ApiInputs, df: pandas.DataFrame):
         logger.error("Additional Columns aside from DeviceId is required to set as MetadataJson.")
         return pandas.DataFrame()
 
-    required_columns.append('MetadataJson')
-
-    # remove not required columns
-    data_frame = data_frame.loc[:, required_columns]
-
     # action request
     headers = api_inputs.api_headers.integration
 
