@@ -17,7 +17,7 @@ Also includes the Automation class which contains helper functions.
 ---------------
 IntegrationTask
 ---------------
-Base class used to create integrations between the Switch Automation platform and other platforms, low-level services,
+Base class used to create integrations between the Switch Automation Platform and other platforms, low-level services,
 or hardware.
 
 Examples include:
@@ -27,7 +27,7 @@ Examples include:
 ---------------------------
 DiscoverableIntegrationTask
 ---------------------------
-Base class used to create integrations between the Switch Automation platform and 3rd party APIs.
+Base class used to create integrations between the Switch Automation Platform and 3rd party APIs.
 
 Similar to the IntegrationTask, but includes a secondary method `run_discovery()` which triggers discovery of available
 points on the 3rd party API and upserts these records to the Switch Platform backend so that the records are available
@@ -62,7 +62,7 @@ Base class used to create data pipelines that are fed via a queue.
 Automation
 ----------
 This class contains the helper methods used to register, deploy, and test the created tasks. Additional helper functions
- for retrieving details of existing tasks on the Switch Automation platform are also included in this module.
+ for retrieving details of existing tasks on the Switch Automation Platform are also included in this module.
 
 """
 import enum
@@ -155,7 +155,7 @@ class Task(ABC):
 class IntegrationTask(Task):
     """Integration Task
 
-    This class is used to create integrations that post data to the Switch Automation platform.
+    This class is used to create integrations that post data to the Switch Automation Platform.
 
 
     Only one of the following methods should be implemented per class, based on the type of integration required:
@@ -218,12 +218,12 @@ class IntegrationTask(Task):
 class DiscoverableIntegrationTask(Task):
     """Discoverable Integration Task
 
-    This class is used to create integrations that post data to the Switch Automation platform from 3rd party APIs that
+    This class is used to create integrations that post data to the Switch Automation Platform from 3rd party APIs that
     have discovery functionality.
 
     The `process()` method should contain the code required to post data for the integration. The `run_discovery()`
     method upserts records into the Integrations table so that end users can configure the points and import as
-    devices/sensors within the Build - Discovery and Selection UI in the Switch Automation platform.
+    devices/sensors within the Build - Discovery and Selection UI in the Switch Automation Platform.
 
     Additional properties are required to be created to support both the discovery functionality & the subsequent
     device/sensor creation from the discovery records.
@@ -243,7 +243,7 @@ class DiscoverableIntegrationTask(Task):
     # @abstractmethod
     # def device_type_definitions(self) -> List[DeviceTypeDefinition]:
     #     """List of DeviceTypeDefinition classes used to create the Device Types available for selection in the
-    #     Build - Discovery & Selection UI in the Switch Automation platform. """
+    #     Build - Discovery & Selection UI in the Switch Automation Platform. """
     #     pass
 
     def check_integration_settings_valid(self, integration_settings: dict):
@@ -308,7 +308,7 @@ class EventWorkOrderTask(Task):
     """Event Work Order Task
 
     This class is used to create work orders in 3rd party systems via tasks that are created in the Events UI of the
-    Switch Automation platform.
+    Switch Automation Platform.
 
     """
 
@@ -338,7 +338,7 @@ class EventWorkOrderTask(Task):
         The definition of the dictionary keys, display labels in Task Insights UI, default value & allowed values
         for the generate_work_order() method's ``integration_settings`` input parameter.
 
-            property_name - the analytics_settings dictionary key
+            property_name - the ``integration_settings`` dictionary key
             display_label - the display label for the given property_name in Task Insights UI
             editor - the editor shown in Task Insights UI
             default_value - default value for this property_name (if applicable)
@@ -349,7 +349,8 @@ class EventWorkOrderTask(Task):
         pass
 
     def check_work_order_input_valid(self, work_order_input: dict):
-        required_work_order_input_keys = set(['EventTaskId', 'Description', 'IntegrationId', 'DueDate'])
+        required_work_order_input_keys = set(['EventTaskId', 'Description', 'IntegrationId', 'DueDate', 'EventLink',
+                                              'EventSummary', 'InstallationId'])
 
         for setting in self.work_order_fields_definition:
             required_work_order_input_keys.add(setting.property_name)
@@ -368,7 +369,7 @@ class EventWorkOrderTask(Task):
             required_integration_keys.add(setting.property_name)
 
         if not required_integration_keys.issubset(set(integration_settings.keys())):
-            logger.error(f'The analytics_setting passed to the task do not contain the required keys: '
+            logger.error(f'The integration_settings passed to the task do not contain the required keys: '
                          f'{required_integration_keys} ')
             return False
         else:
@@ -387,14 +388,21 @@ class EventWorkOrderTask(Task):
         this method will contain the following keys:
 
         - `EventTaskId`
-            - unique identifier (uuid.UUID) for the given record in the Switch Automation platform
+            - unique identifier (uuid.UUID) for the given record in the Switch Automation Platform
         - `Description`
             - free text field describing the work order to be generated.
         - `IntegrationId`
             - if linked to an existing work order in the 3rd party API, this will contain that system's identifier for
             the workorder. If generating a net new workorder, this field will be null (None).
         - `DueDate`
-            - The due date for the work order as set in the Switch Automation platform.
+            - The due date for the work order as set in the Switch Automation Platform.
+        - `EventLink`
+            - The URL link to the given Event in the Switch Automation Platform UI.
+        - `EventSummary`
+            - The Summary text associated with the given Event in the Switch Automation Platform UI.
+        - `InstallationId`
+            - The unique identifier of the site in the Switch Automation Platform that the work order is associated
+            with.
 
         Parameters
         ----------
@@ -404,7 +412,7 @@ class EventWorkOrderTask(Task):
             Any settings required to be passed to the integration to run. For example, username & password, api key,
             auth token, etc.
         work_order_input : dict
-            The work order defined by the task created in the Events UI of the Switch Automation platform. To be sent
+            The work order defined by the task created in the Events UI of the Switch Automation Platform. To be sent
             to 3rd party system for creation.
         """
 
@@ -414,7 +422,7 @@ class EventWorkOrderTask(Task):
 class QueueTask(Task):
     """Queue Task
 
-    This class is used to create integrations that post data to the Switch Automation platform using a Queue as the
+    This class is used to create integrations that post data to the Switch Automation Platform using a Queue as the
     data source.
 
     Only one of the following methods should be implemented per class, based on the type of integration required:
